module savegloval_define

!-- setting global variables

!-- parameter

  integer, parameter :: nzlim=1000
  integer, parameter :: subn=26
  character(1), dimension(subn), parameter ::  &
  &                                      subhead=(/'a','b','c','d','e','f',  &
  &                                                'g','h','i','j','k','l',  &
  &                                                'm','n','o','p','q','r',  &
  &                                                's','t','u','v','w','x',  &
  &                                                'y','z'/)
  character(8), parameter :: rest='+restime'
  character(6), parameter :: restn='+restn'
  character(5), parameter :: respsir='psikr'
  character(5), parameter :: respsii='psiki'
  character(3), parameter :: reszetar='zkr'
  character(3), parameter :: reszetai='zki'
  character(7), parameter :: res_nbm_optr='nbmoptr'
  character(7), parameter :: res_nbm_opti='nbmopti'

  character(6), parameter :: resumblr='ukmblr'
  character(6), parameter :: resumbli='ukmbli'
  character(6), parameter :: resvmblr='vkmblr'
  character(6), parameter :: resvmbli='vkmbli'
  character(8), parameter :: res_mbl_optur='mbloptur'
  character(8), parameter :: res_mbl_optui='mbloptui'
  character(8), parameter :: res_mbl_optvr='mbloptvr'
  character(8), parameter :: res_mbl_optvi='mbloptvi'

!-- namelist variables

  integer, save :: npe     ! process number of MPI (mod(nz,npe)==0)
  integer, save :: nx      ! grid number of x direction
  integer, save :: ny      ! grid number of y direction
  integer, save :: nz      ! grid number of z direction
  integer, save :: hxnt    ! trancation wave number of x direction
  integer, save :: hynt    ! trancation wave number of y direction
  integer, save :: jxnt    ! grid number of x direction calculating integration
  integer, save :: jynt    ! grid number of y direction calculating integration
  integer, save :: nt      ! grid number of time
  integer, save :: dmpstp  ! output interval
  integer, save :: i_adj   ! time step of additional time integration for initialization in MBL
  integer, save :: smt_mbl_opt ! option of smoothing in MBL (0: no, 1: yes)
  integer, save :: ndiff_mbl ! Laplacian order of numerical diffusion in horizontal coordinates

  integer, save :: nxi     ! grid number of x direction
  integer, save :: nyi     ! grid number of y direction
  integer, save :: restp   ! restart output interval
  integer, save :: resopt  ! restart option
  integer, save :: force_type  ! Type of the stretching forcing

  double precision, save :: Lx       ! calculating domain of x direction
  double precision, save :: Ly       ! calculating domain of y direction
  double precision, save :: dt       ! grid interval of time
  double precision, save :: nu       ! diffusion coefficient in NBM
  double precision, save :: mu       ! damping coefficient
  double precision, save :: zetam    ! maximum vorticity of the outer eyewall
  double precision, save :: sth_thres_zeta 
                                     ! threshold of vorticity for stretching
  double precision, save :: cent_lat ! centered latitude on the f-plane
  double precision, save :: KH_mbl   ! horizontal diffusion coefficient in MBL
  double precision, save :: K_mbl    ! vertical diffusion coefficient in MBL

  character(3), save :: time_flag    ! time integration type
  character(1000), save :: foname    ! output file name
  character(1000), save :: ininame   ! input file name
  character(1000), save :: sth_fname ! stretching coefficient file name
  character(1000), save :: resfname  ! restart file name
  character(100), save :: inix       ! X-coordinate name
  character(100), save :: iniy       ! y-coordinate name
  character(100), save :: iniz       ! initial zeta name
  character(100), save :: sth_vname  ! stretching coefficient variable name
  character(100), save :: inixd      ! X-coordinate name for double
  character(100), save :: iniyd      ! y-coordinate name for double

  logical, save :: flag_stretch      ! Flag for stretching forcing
  logical, save :: flag_sthmod       ! Flag for stretching forcing
  logical, save :: flag_fric         ! Flag for friction
  logical, save :: calc_nbm_flag     ! Flag for calculation in NBM
  logical, save :: calc_mbl_flag     ! Flag for calculation in MBL

!-- others variables

  integer, save :: iz_str  ! first index in z of each MPI process
  integer, save :: iz_end  ! last index in z of each MPI process
  integer, save :: nzp     ! nz/npe
  integer, save :: kxnt    ! 2*hxnt+1
  integer, save :: kynt    ! 2*hynt+1
  integer, save :: nrt     ! restart time step
  double precision, save :: xmin    ! minimum x-coordinate
  double precision, save :: ymin    ! minimum x-coordinate
  double precision, save :: dx      ! grid interval of x direction
  double precision, save :: dy      ! grid interval of y direction
  double precision, save :: f0      ! Coriolis parameter
  double precision, save :: rho0    ! Air density
  real, save :: restime             ! restart starting time
  double precision, allocatable, dimension(:), save :: xi ! array of x direction
  double precision, allocatable, dimension(:), save :: yi ! array of y direction
  real, allocatable, dimension(:), save :: xd ! array of x direction
  real, allocatable, dimension(:), save :: yd ! array of y direction
  real, allocatable, dimension(:), save :: zd ! array of z direction
  real, allocatable, dimension(:), save :: zd_pe ! array of z direction
  double precision, allocatable, dimension(:), save :: xj ! array of x direction
  double precision, allocatable, dimension(:), save :: yj ! array of y direction
  double precision, allocatable, dimension(:), save :: x  ! array of x direction
  double precision, allocatable, dimension(:), save :: y  ! array of y direction
  double precision, allocatable, dimension(:), save :: z  ! array of z direction
  double precision, allocatable, dimension(:), save :: z_pe ! array of z direction
  double precision, dimension(nzlim), save :: z_mbl  ! array of z direction
!ORG  double precision, allocatable, dimension(:), save :: t  ! array of time
  double precision, allocatable, dimension(:,:), save :: cr  ! stretching coefficient
  double precision, allocatable, dimension(:,:), save :: cr_isp  ! stretching coefficient for ISPACK
  double precision, allocatable, dimension(:,:), save :: zinit_isp  ! initial vorticity for ISPACK

!-- rotate arrays for fft

  complex(kind(0d0)), save :: basezeta

!-- rotate arrays for ISPACK

  integer, allocatable, dimension(:), save :: ITJR
  integer, allocatable, dimension(:), save :: ITIR
  integer, allocatable, dimension(:), save :: ITJJ
  integer, allocatable, dimension(:), save :: ITIJ
  double precision, allocatable, dimension(:), save :: TJR
  double precision, allocatable, dimension(:), save :: TIR
  double precision, allocatable, dimension(:), save :: TJJ
  double precision, allocatable, dimension(:), save :: TIJ

!-- MPI variables

  integer, save :: MY_RANK   ! this process number
  integer, save :: PETOT     ! total process number

!-- for debug

  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: HADVud
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: HADVvd
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: VADVud
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: VADVvd
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: HDIFFud
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: HDIFFvd
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: VDIFFud
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: VDIFFvd
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: CORILud
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: CORILvd
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: PGRADud
  complex(kind(0d0)), allocatable, dimension(:,:,:), save :: PGRADvd

end module savegloval_define
