program other_2d
! 複数のテキストデータから時系列図を作成するプログラム
! リスト作成の際は 00Z で始まり, 00Z で終わるように作成すること.
! 足らない部分は unknown で補う.

  use dcl
  use Dcl_Automatic
  use file_operate
  use basis
  use Statistics

  implicit none

!-- integer
  integer :: nt       ! 時間方向の格子数
  integer :: nz       ! z 方向の格子数
  integer :: nf       ! conv_dat 用データ数
  integer :: IWS      ! 描画デバイス
  integer :: days     ! 日数
  integer :: i_counter    ! 一時カウンター
  integer :: i_counter_c  ! 一時カウンター
  integer :: snum, cnum  ! カラーおよび等値線の数
  integer :: skip_num  ! 先頭読み飛ばし行数
  integer, allocatable, dimension(:) :: year_v    ! 年
  integer, allocatable, dimension(:) :: month_v   ! 月
  integer, allocatable, dimension(:) :: day_v     ! 日
  integer, allocatable, dimension(:) :: hour_v    ! 時間
  integer, allocatable, dimension(:) :: inter_z ! 内挿用高度配列
  integer, allocatable, dimension(:) :: i_flag  ! 一時配列
  integer, allocatable, dimension(:) :: o_flag  ! 一時配列
  integer, allocatable, dimension(:) :: c_flag  ! 一時配列
!-- tmp
  integer :: nttmp, i, j, k, i_undef
  integer :: col_num, idz_val, idc_val, ids_val, idu_val, idv_val

!-- real
  real :: z_bot       ! 描画の下端
  real :: z_top       ! 描画の上端
  real :: dz          ! 鉛直方向の描画間隔 [m]
  real :: dvy         ! vy の間隔
  real :: smax, smin, cmax, cmin  ! カラー, 等値線の上限
  real, allocatable, dimension(:) :: time  ! 時間
  real, allocatable, dimension(:) :: z     ! 高度座標
  real, allocatable, dimension(:) :: z_inv ! 高度座標
  real, allocatable, dimension(:) :: vx    ! viewport 配列
  real, allocatable, dimension(:,:) :: vy  ! viewport 配列
  real, allocatable, dimension(:,:) :: val_f        ! 計算用配列
  real, allocatable, dimension(:,:) :: val_inter    ! 計算用変数配列
  real, allocatable, dimension(:,:) :: tline        ! 計算用変数配列
  real, allocatable, dimension(:,:) :: yline        ! 計算用変数配列
  real, allocatable, dimension(:,:,:) :: val_draw   ! 描画用変数配列
  real :: undef, undef_1d
!-- tmp
  real, dimension(2) :: s_int, c_int

!-- character
  character(80) :: flist         ! ファイルリスト名
  character(80) :: dc_val        ! ファイルリストの描画フラグ
  character(80) :: conv_dat      ! 時系列データ
  character(80) :: conv_list     ! 時系列データ
  character(80) :: title_txt     ! タイトル名
  character(10) :: dc_flag       ! カラムデータフラグ
  character(100) :: z_title      ! 縦軸のタイトル
  character(20) :: sform, cform  ! フォーマット
  character(80), allocatable, dimension(:,:) :: fname  ! ファイル名
  character(20), allocatable, dimension(:,:) :: val_w  ! テキストデータ
  character(100) :: skip_char    ! 先頭ヘッダー識別子
                                 ! 読み飛ばし行数が変動するフォーマットの際使用.
  character(70) :: sysfont
!-- tmp
  character(80), allocatable, dimension(:) :: dummy_read
  character(80) :: tmp_c
  character(1) :: skip_flag

!-- logical
  logical :: inv_flag  ! 軸を逆転させるかのフラグ.
                       ! 格納されている高度座標を逆転させるときは .true.
  logical :: fflag, cont_flag, shade_flag, u_flag, v_flag

!-- type
  type(dtime) :: start_day, end_day
  type(dcl_date) :: stime

!-- namelist reading
  namelist /input_2d /flist, dc_val, dz, z_bot,  &
  &                  z_top, IWS, title_txt,  &
  &                  cmin, cmax, smin, smax, cnum, snum, cform, sform,  &
  &                  undef, skip_num, skip_char, inv_flag, z_title, sysfont
  read(5,nml=input_2d)

  namelist /input_1d /conv_dat, conv_list, undef_1d
  read(5,nml=input_1d)

  fflag=.false.

  if(trim(skip_char)=='')then
     skip_flag='n'
  else
     skip_flag='c'
  end if

  nz=int((z_top-z_bot)/dz)+1
  nt=line_number_counter( trim(flist) )
  s_int=(/smin, smax/)
  c_int=(/cmin, cmax/)
  col_num=len_trim(dc_val)

  allocate(fname(2,nt))
  allocate(time(nt))
  allocate(z(nz))
  allocate(z_inv(nz))
  allocate(inter_z(nz))
  allocate(val_inter(col_num,nz))
  allocate(val_draw(col_num,nt,nz))
  allocate(year_v(nt))
  allocate(month_v(nt))
  allocate(day_v(nt))
  allocate(hour_v(nt))

!-- DCL set
  if(len_trim(adjustl(sysfont))/=0)then
     call SGISET( 'IFONT', 1 )
     call SWLSET( 'LSYSFNT', .true. )
     write(*,*) "This drawing mode is sysfont."
  else
     call SGISET( 'IFONT', 2 )
     write(*,*) "This drawing mode is dclfont."
  end if

  call UZFACT(0.8)
  call DclSetParm( 'ENABLE_CONTOUR_MESSAGE', .false. )
  CALL GLRSET( 'RMISS', undef )
  CALL GLLSET( 'LMISS', .TRUE. )

!-- drawing value separate
  cont_flag=.false.
  shade_flag=.false.
  u_flag=.false.
  v_flag=.false.

  do i=1,col_num
     if(dc_val(i:i)=='z')then
        idz_val=i
     end if
     if(dc_val(i:i)=='c')then
        idc_val=i
        cont_flag=.true.
     end if
     if(dc_val(i:i)=='s')then
        ids_val=i
        shade_flag=.true.
     end if
     if(dc_val(i:i)=='u')then
        idu_val=i
        v_flag=.true.
     end if
     if(dc_val(i:i)=='v')then
        idv_val=i
        v_flag=.true.
     end if
  end do

  call read_file_text( trim(flist), 2, nt, fname )

  do i=1,nt
     tmp_c=trim(fname(2,i))
     year_v(i)=c2i_convert( trim(tmp_c(1:4)) )
     month_v(i)=c2i_convert( trim(tmp_c(5:6)) )
     day_v(i)=c2i_convert( trim(tmp_c(7:8)) )
     hour_v(i)=c2i_convert( trim(tmp_c(9:10)) )
  end do

  start_day%year_d=year_v(1)
  start_day%month_d=month_v(1)
  start_day%day_d=day_v(1)
  start_day%hour_d=hour_v(1)
  start_day%min_d=0
  start_day%sec_d=0
  end_day%year_d=year_v(nt)
  end_day%month_d=month_v(nt)
  end_day%day_d=day_v(nt)
  end_day%hour_d=hour_v(nt)
  end_day%min_d=0
  end_day%sec_d=0
  days=counter_day( start_day, end_day )-1
  stime%year=year_v(1)
  stime%month=month_v(1)
  stime%day=day_v(1)

  do i=1,nz
     z(i)=z_bot+dz*real(i-1)
  end do
  if(inv_flag.eqv..true.)then
     do i=1,nz
        z_inv(i)=z(nz-i+1)
     end do
  end if
  do i=1,nt
     end_day%year_d=year_v(i)
     end_day%month_d=month_v(i)
     end_day%day_d=day_v(i)
     time(i)=real(counter_day( start_day, end_day ))  &
  &         +real(hour_v(i)-hour_v(1))/24.0-1.0
  end do

!-- time loop
!-- 1. reading values from text file
!-- 2. interpolating to grid point
!-- 3. calculating each value

  do i=1,nt
     if(trim(fname(1,i))/='unknown')then
        if(skip_flag(1:1)=='c')then
           nttmp=line_number_counter( trim(fname(1,i)) )
           allocate(dummy_read(nttmp))
           call read_file_text( trim(fname(1,i)), 1, nttmp, dummy_read )
           do j=1,nttmp-1
              if(trim(dummy_read(j))==trim(skip_char))then
                 skip_num=j
                 deallocate(dummy_read)
                 exit
              end if
           end do
        end if

        nttmp=line_number_counter( trim(fname(1,i)) )-skip_num
        allocate(val_w(col_num,nttmp))
        allocate(val_f(col_num,nttmp))

        call read_file_text( trim(fname(1,i)), col_num, nttmp, val_w,  &
  &                          skip=skip_num )

     !-- convert type from character to float
        do k=1,nttmp
           do j=1,col_num
              val_f(j,k)=c2r_convert( val_w(j,k) )
           end do
        end do

     !-- calculating the interpolating points
        do k=1,nz
           call interpo_search_1d( val_f(idz_val,:), z(k), inter_z(k),  &
  &                                undeff=int(undef) )
        end do

     !-- calculating each value
        do k=1,nz
           do j=1,col_num
              if(j/=idz_val)then
                 if(inter_z(k)/=int(undef).and.inter_z(k)/=nttmp)then
                    if(val_f(j,inter_z(k))==undef.or.  &
  &                    val_f(j,inter_z(k)+1)==undef)then
                       val_inter(j,k)=undef
                       exit
                    end if

                    call interpolation_1d( val_f(idz_val,inter_z(k):inter_z(k)+1),  &
  &                                        val_f(j,inter_z(k):inter_z(k)+1),  &
  &                                        z(k), val_inter(j,k) )
                 else
                    val_inter(j,k)=undef
                 end if
              end if
           end do
        end do

        deallocate(val_w)
        deallocate(val_f)

     else
        val_inter=undef
     end if

     do j=1,col_num
        do k=1,nz
           if(inv_flag.eqv..true.)then
              val_draw(j,i,k)=val_inter(j,nz-k+1)
           else
              val_draw(j,i,k)=val_inter(j,k)
           end if
        end do
     end do

  end do

!-- DCL part
!-- まず, 高度-時系列断面
  call color_setting( snum, s_int, col_min=15, col_max=85,  &
  &                   min_tab=10999, max_tab=99999 )

  call DclOpenGraphics(IWS)

  if(len_trim(adjustl(sysfont))/=0)then
!     CALL SWSLFT(sysfont)
     CALL SWCSET('FONTNAME', sysfont)
  end if

  if(v_flag.eqv..false.)then
     if(inv_flag.eqv..false.)then
        call Dcl_2D_Cont_Shade_Calendar( trim(title_txt),  &
  &          time, z, val_draw(idc_val,:,:), val_draw(ids_val,:,:),  &
  &          c_int, s_int,  &
  &          (/'            ', trim(z_title)/), stime, days,  &
  &          (/trim(cform), trim(sform)/),  &
  &          c_num=(/cnum, snum/), trigleg='a' )
     else
        call Dcl_2D_Cont_Shade_Calendar( trim(title_txt),  &
  &          time, z_inv, val_draw(idc_val,:,:), val_draw(ids_val,:,:),  &
  &          c_int, s_int,  &
  &          (/'            ', trim(z_title)/), stime, days,  &
  &          (/trim(cform), trim(sform)/),  &
  &          c_num=(/cnum, snum/), trigleg='a' )
     end if
  else
     if(size(time)<20)then
        if(inv_flag.eqv..false.)then
           call Dcl_2D_Cont_Shade_vec_Calendar( trim(title_txt),  &
  &             time, z, val_draw(idc_val,:,:), val_draw(ids_val,:,:),  &
  &             val_draw(idu_val,:,:), val_draw(idv_val,:,:), (/nt,20/),  &
  &             c_int, s_int,  &
  &             (/'            ', trim(z_title)/), stime, days,  &
  &             (/trim(cform), trim(sform)/),  &
  &             c_num=(/cnum, snum/), trigleg='a',  &
  &             unitv=.true., vfact=(/2.0e-3, 2.0e-3/),  &
  &             unit_fact=(/5.0e-2, 5.0e-2/),  &
  &             unit_fact_sign=.false., unit_title=(/'U = 25m/s', ''/) )
        else
           call Dcl_2D_Cont_Shade_vec_Calendar( trim(title_txt),  &
  &             time, z_inv, val_draw(idc_val,:,:), val_draw(ids_val,:,:),  &
  &             val_draw(idu_val,:,:), val_draw(idv_val,:,:), (/nt,20/),  &
  &             c_int, s_int,  &
  &             (/'            ', trim(z_title)/), stime, days,  &
  &             (/trim(cform), trim(sform)/),  &
  &             c_num=(/cnum, snum/), trigleg='a',  &
  &             unitv=.true., vfact=(/2.0e-3, 2.0e-3/),  &
  &             unit_fact=(/5.0e-2, 5.0e-2/),  &
  &             unit_fact_sign=.false., unit_title=(/'U = 25m/s', ''/) )
        end if
     else
        if(inv_flag.eqv..false.)then
           call Dcl_2D_Cont_Shade_vec_Calendar( trim(title_txt),  &
  &             time, z, val_draw(idc_val,:,:), val_draw(ids_val,:,:),  &
  &             val_draw(idu_val,:,:), val_draw(idv_val,:,:), (/nt,20/),  &
  &             c_int, s_int,  &
  &             (/'            ', trim(z_title)/), stime, days,  &
  &             (/trim(cform), trim(sform)/),  &
  &             c_num=(/cnum, snum/), trigleg='a',  &
  &             unitv=.true., vfact=(/2.0e-3, 2.0e-3/),  &
  &             unit_fact=(/5.0e-2, 5.0e-2/),  &
  &             unit_fact_sign=.false., unit_title=(/'U = 25m/s', ''/) )
        else
           call Dcl_2D_Cont_Shade_vec_Calendar( trim(title_txt),  &
  &             time, z_inv, val_draw(idc_val,:,:), val_draw(ids_val,:,:),  &
  &             val_draw(idu_val,:,:), val_draw(idv_val,:,:), (/nt,20/),  &
  &             c_int, s_int,  &
  &             (/'            ', trim(z_title)/), stime, days,  &
  &             (/trim(cform), trim(sform)/),  &
  &             c_num=(/cnum, snum/), trigleg='a',  &
  &             unitv=.true., vfact=(/2.0e-3, 2.0e-3/),  &
  &             unit_fact=(/5.0e-2, 5.0e-2/),  &
  &             unit_fact_sign=.false., unit_title=(/'U = 25m/s', ''/) )
        end if
     end if
  end if

!-- 以降, 対流パラメータ変数の時系列データ書き出し

  deallocate(fname)
  deallocate(year_v)
  deallocate(month_v)
  deallocate(day_v)
  deallocate(hour_v)
  deallocate(time)

  if(trim(conv_dat(1:1))/='')then
     nf=len_trim(conv_list)
     nt=line_number_counter( trim(conv_dat) )

     allocate(fname(nf,nt))
     allocate(year_v(nt-2))
     allocate(month_v(nt-2))
     allocate(day_v(nt-2))
     allocate(hour_v(nt-2))
     allocate(i_flag(3))
     allocate(o_flag(nf))
     allocate(c_flag(nf))
     allocate(time(nt-2))
     allocate(tline(nt,nf))
     allocate(yline(nt,nf))

     call read_file_text( trim(conv_dat), nf, nt, fname )
     i_counter=0
     i_counter_c=0

     do i=1,nf
        select case(conv_list(i:i))
        case('t')
           i_flag(1)=i
        case('o')
           i_counter=i_counter+1
           o_flag(i_counter)=i
        end select
     end do

     do i=1,nt-2
        tmp_c=trim( adjustl( fname(i_flag(1),i+2) ) )
        year_v(i)=c2i_convert( trim(tmp_c(1:4)) )
        month_v(i)=c2i_convert( trim(tmp_c(5:6)) )
        day_v(i)=c2i_convert( trim(tmp_c(7:8)) )
        hour_v(i)=c2i_convert( trim(tmp_c(9:10)) )
     end do

     start_day%year_d=year_v(1)
     start_day%month_d=month_v(1)
     start_day%day_d=day_v(1)
     end_day%year_d=year_v(nt-2)
     end_day%month_d=month_v(nt-2)
     end_day%day_d=day_v(nt-2)
     days=counter_day( start_day, end_day )-1
     stime%year=year_v(1)
     stime%month=month_v(1)
     stime%day=day_v(1)

     do i=1,nt-2
        end_day%year_d=year_v(i)
        end_day%month_d=month_v(i)
        end_day%day_d=day_v(i)
        time(i)=real(counter_day( start_day, end_day ))+real(hour_v(i))/24.0-1.0
     end do

     if(i_counter>0)then

     !-- vx を i_counter で分ける.
        allocate(vx(2))
        allocate(vy(i_counter,2))
        vx=(/0.2, 0.95/)
        dvy=(0.8-0.1*real(i_counter-1))/real(i_counter)
        vy(1,1)=0.1
        vy(1,2)=vy(1,1)+dvy
        if(i_counter>1)then
           do i=2,i_counter
              vy(i,1)=vy(i-1,2)+0.1
              vy(i,2)=vy(i,1)+dvy
           end do
        end if

        do i=1,nt-2
           do j=1,i_counter
              tmp_c=trim( adjustl( fname(o_flag(j),i+2) ) )
              tline(i,j)=time(i)
              yline(i,j)=c2r_convert( trim(tmp_c))
           end do
        end do
      
        do j=1,i_counter
           i_undef=1
           do i=1,nt-2
              if(yline(i,j)/=undef)then
                 exit
              else
                 i_undef=i_undef+1
              end if
           end do

           if(j>1)then
              fflag=.true.
           end if
           tmp_c=trim(fname(o_flag(j),1))//  &
  &             '('//trim(fname(o_flag(j),2))//')'

           call Dcl_PL_Calendar( 'l', '',   &
  &             tline(i_undef:nt-2,j:j), yline(i_undef:nt-2,j:j),  &
  &             tline(i_undef:nt-2,j:j), yline(i_undef:nt-2,j:j),  &
  &             (/trim(tmp_c), trim(tmp_c)/),  &
  &             stime, days, viewx_int=vx, viewy_int=vy(j,:),  &
  &             no_frame=fflag )
        end do

     end if

     if(i_counter_c>0)then

        do i=1,nt-2
           do j=1,i_counter_c
              tmp_c=trim(fname(o_flag(j),i+2))
              tline(i,j)=time(i)
              yline(i,j)=c2r_convert( trim(tmp_c))
           end do
        end do
      
        tmp_c=''
        do j=1,i_counter_c
           tmp_c=trim(tmp_c)//', '//trim(fname(c_flag(j),1))
        end do
      
        i_undef=1
        do i=1,nt-2
           if(yline(i,j)/=undef)then
              exit
           else
              i_undef=i_undef+1
           end if
        end do
        call Dcl_PL_Calendar( 'l', '',  &
  &          tline(i_undef:nt-2,1:c_flag(j)), yline(i_undef:nt-2,1:c_flag(j)),  &
  &          tline(i_undef:nt-2,1:c_flag(j)), yline(i_undef:nt-2,1:c_flag(j)),  &
  &          (/'', trim(tmp_c)/), stime, days )

     end if

  end if

  call DclCloseGraphics

end program
